#!/usr/bin/env python2
from __future__ import with_statement
from scipy import *
import numpy as np
from numpy import min, max
import pylab
import sys, os, re, glob
import load, getChi
from scipy import linspace, polyval, polyfit
import scipy.stats 
from essentiality import seqLoad, getEnergy, hamming, loadSites, loadEssentialityData

cc = scipy.stats.spearmanr
sem = lambda x: scipy.stats.sem(x) if len(x) > 1 else nan
mean = lambda x: np.mean(x) if len(x) > 0 else nan
var = lambda x: np.var(x) if len(x) > 0 else nan

outdir = 'results'

def shiftEM(em):
    return em - mean(em,1)[:,newaxis]

def main():
    pylab.rcParams['figure.subplot.bottom'] = 0.15

    essential, tested, rates, expr, cons = loadEssentialityData()
    
    pvals = []
    tfs = []
    for emfile in glob.glob(os.path.join(sys.argv[1], '*')):

        em = loadtxt(emfile)
        tf = os.path.basename(emfile)
        tfs.append(tf)
        tffile = os.path.join(sys.argv[2], tf)
        if not os.path.exists(tffile):
            print "skipping" + tffile
            continue
    
        try:
            outf = open(os.path.join(outdir, tf, 'tssdist.txt'), 'wt')
        except:
            continue

        #load gene to  TFBS map
        regsites = loadSites(tffile)
        seqs = array([seqLoad(sites['recomputed'].strip()) for gene, pos, dist, sites in regsites])
        energies = load.getEnergies(seqs, em)
        emin, emax = min(energies), max(energies)
        bins = arange(emin, emax+1e-5, (emax-emin)/10.0)

        essentiale = []
        nonessentiale = []
        singleTF = []
        unknowne = []
        data = []
        for e,(gene, pos, dist, sites) in zip(energies, regsites):
            data.append((e,float(dist)))
            if gene in essential:
                essentiale.append((e,dist))
            elif gene in tested:
                nonessentiale.append((e,dist))
            else:
                unknowne.append((e,dist))


        data = array(data)    
        essentiale = array(essentiale, dtype=float) if len(essentiale) != 0 else []
        nonessentiale = array(nonessentiale, dtype=float) if len(nonessentiale) != 0 else []

        corr = cc(data[:,0], data[:,1])
        print >>outf, corr[0], corr[1]
        print >>outf, mean(essentiale), mean(nonessentiale), len(essentiale), len(nonessentiale)
        pvals.append(cc(data[:,0], data[:,1]))

        outf.close()

        pylab.figure()
        pylab.plot(data[:,0], data[:,1], '.')
        pylab.xlabel('Energy')
        pylab.ylabel('Distance to TSS')
        pylab.tight_layout()
        pylab.savefig(os.path.join(outdir, tf, 'tssVe.svg'))
        pylab.close()
        
        if len(essentiale) != 0 and len(nonessentiale) != 0:
            pylab.figure()
            pylab.hist([essentiale, nonessentiale], bins=20)
            pylab.xlabel('TSS distance')
            pylab.ylabel('#')
            pylab.tight_layout()
            pylab.legend(['essential genes', 'nonessential genes'])
            pylab.savefig(os.path.join(outdir, tf, 'tss_dist.svg'))
            pylab.close()

    pvals = array(pvals)
    for tf, pval in zip(tfs, pvals):
        print "{}: {}    {:.4f}".format(tf, pval[0], pval[1])
    print mean(pvals[:,1])


if __name__ == '__main__':
    main()

