#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "loadData.h"

char *readFile(char *filename, int *fileLen){
    FILE * f;
    long flen;
    char * buffer;

    f = fopen(filename, "rb");
    if(f == NULL){
        return NULL;
    }

    // obtain file size:
    fseek(f, 0, SEEK_END);
    flen = ftell(f);
    rewind(f);

    buffer = (char*)malloc(sizeof(char)*(flen+1));
    size_t count = fread(buffer, 1, flen, f);

    buffer[flen] = '\0';//null terminate
    *fileLen = flen+1;
    
    fclose(f);
    
    return buffer;
}

void expandBuffer(double **buf, int size, int scale){
    double *newbuf = malloc(sizeof(double)*scale*size);
    memcpy(newbuf, *buf, sizeof(double)*size);
    free(*buf);
    *buf = newbuf;
}

void readData(char *datafilename, int *nx_i, double **x_i, double **background_i, int *nEn_i, double **energies_i, double **bgp_i){
    int i,j;

    int filelen;
    char *filedata = readFile(datafilename, &filelen);
    
    char *line;
    char *word;
    char *lineptr;
    char *wordptr;
    
    //first line should have a list of energies
    line = strtok_r(filedata, "\n", &lineptr);

    //get energies
    int enbuffersize = 4096;
    double *enbuffer = malloc(sizeof(double)*enbuffersize);
    word = strtok_r(line, " ", &wordptr);
    int nEn = 0;
    while(word != NULL){
        if(nEn > enbuffersize){
            expandBuffer(&enbuffer, enbuffersize, 4);
            enbuffersize *= 4;
        }
        enbuffer[nEn] = atof(word);
        nEn += 1;
        word = strtok_r(NULL, " ", &wordptr);
    }
    double *energies = malloc(sizeof(double)*nEn);
    memcpy(energies, enbuffer, sizeof(double)*nEn);
    free(enbuffer);
    
    //get sequence background probabilities
    //assumes it's the same size as energies
    double *seqbgp = malloc(sizeof(double)*nEn);
    line = strtok_r(NULL, "\n", &lineptr);
    word = strtok_r(line, " ", &wordptr);
    i = 0;
    while(word != NULL){
        if(i >= nEn){
            printf("ERROR: wrong # of sequence background probabilities\n");
            exit(1);
        }
        seqbgp[i] = atof(word);
        i += 1;
        word = strtok_r(NULL, " ", &wordptr);
    }
    
    //read remaining lines to get x
    int xbuffersize = 4096;
    double *xbuffer = malloc(sizeof(double)*xbuffersize);
    double *bgbuffer = malloc(sizeof(double)*xbuffersize);
    int nx = 0;
    line = strtok_r(NULL, "\n", &lineptr);
    while(line != NULL){
        if(nx > xbuffersize){
            expandBuffer(&xbuffer, xbuffersize, 4);
            expandBuffer(&bgbuffer, xbuffersize, 4);
            xbuffersize *= 4;
        }
        word = strtok_r(line, " ", &wordptr);
        xbuffer[nx] = atof(word);
        word = strtok_r(NULL, " ", &wordptr);
        bgbuffer[nx] = atof(word);
        nx += 1;
        line = strtok_r(NULL, "\n", &lineptr);
    }
    double *x = malloc(sizeof(double)*nx);
    memcpy(x, xbuffer, sizeof(double)*nx);
    free(xbuffer);
    double *background = malloc(sizeof(double)*nx);
    memcpy(background, bgbuffer, sizeof(double)*nx);
    free(bgbuffer);

    *nx_i = nx;
    *x_i = x;
    *background_i = background;

    *nEn_i = nEn;
    *energies_i = energies;
    *bgp_i = seqbgp;
}
