#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_min.h>
#include "loadData.h"
#include "logLike.h"

typedef struct {
    double min, max, dx;
    int n;
}Range;

Range makeRange(double min, double max, double dx){
    Range new;
    new.min = min;
    new.max = max;
    new.dx = dx;
    new.n = (int)((max-min)/dx);
    return new;
}

double getBin(Range r, int i){
    return r.min + r.dx*i;
}

int main(int argc, char *argv[]){    
    int i,j;
    gsl_set_error_handler_off();

    int nx;
    double *x, *background;
    int nEn;
    double *energies, *seqbgp;
    readData(argv[1], &nx, &x, &background, &nEn, &energies, &seqbgp);
    init(nx, x, background, nEn, energies, seqbgp);

    if(nparam != 4){
        printf("Need to compile with Nonlethal fitness");
    }
    gsl_vector *param;
    param = gsl_vector_alloc(nparam);

    Range tc = makeRange(-5,5,0.1);
    Range lN = makeRange(-3,5,0.1);
    Range mur = makeRange(-20,0,0.2);
    Range lB = makeRange(-1,1,0.1);

    int tci, lNi, muri, lBi;
    double mu, B, c, N;

    double *bestBs = malloc(sizeof(double)*lB.n);
    double *bestmus = malloc(sizeof(double)*mur.n);
    double *bestnf0s = malloc(sizeof(double)*lN.n*tc.n);
    for(i = 0; i < lB.n; i++){
        bestBs[i] = -INFINITY;
    }
    for(i = 0; i < mur.n; i++){
        bestmus[i] = -INFINITY;
    }
    for(i = 0; i < lN.n*tc.n; i++){
        bestnf0s[i] = -INFINITY;
    }

    double bestc, bestn, bestmu, bestb;
    double bestOfAll = -INFINITY;

    for(tci = 0; tci < tc.n; tci++){
        c = (tanh(getBin(tc,tci))+1)/2;
        if( c > 0.99){
            break;
        }
    }
    int targetC = tci;
    double bestc_constr, bestn_constr, bestmu_constr, bestb_constr;
    double bestOfAll_constr = -INFINITY;

    for(tci = 0; tci < tc.n; tci++){
        c = (tanh(getBin(tc,tci))+1)/2;
        for(lNi = 0; lNi < lN.n; lNi++){
            N = pow(10, getBin(lN,lNi));
            for(muri = 0; muri < mur.n; muri++){
                mu = getBin(mur, muri);
                for(lBi = 0; lBi <= lB.n; lBi++){
                    B = pow(10,getBin(lB,lBi));

                    gsl_vector_set(param, 0, N);
                    gsl_vector_set(param, 1, mu);
                    gsl_vector_set(param, 2, B);
                    gsl_vector_set(param, 3, c);
                    mapParam(param);
                    double val = logLikelihood(param);

                    if(val > bestOfAll){
                        bestc = c;
                        bestn = N;
                        bestmu = mu;
                        bestb = B;
                        bestOfAll = val;
                    }
                    if(val > bestBs[lBi]){
                        bestBs[lBi] = val;
                    }
                    if(val > bestmus[muri]){
                        bestmus[muri] = val;
                    }
                    if(val > bestnf0s[lNi + lN.n*tci]){
                        bestnf0s[lNi + lN.n*tci] = val;
                    }

                    if(tci == targetC){
                        if(val > bestOfAll_constr){
                            bestc_constr = c;
                            bestn_constr = N;
                            bestmu_constr = mu;
                            bestb_constr = B;
                            bestOfAll_constr = val;
                        }
                    
                    }
                }
            }
        }
    }
    
    printf("%.15g %.15g %.15g %.15g  %.15g\n\n", bestn, bestmu, bestb, bestc, bestOfAll);
    printf("%.15g %.15g %.15g %.15g  %.15g\n\n", bestn_constr, bestmu_constr, bestb_constr, bestc_constr, bestOfAll_constr);

    for(i = 0; i < lB.n; i++){
        printf("%.15g %.15g\n", pow(10,getBin(lB,i)), bestBs[i]);
    }
    printf("\n\n");
    for(i = 0; i < mur.n; i++){
        printf("%.15g %.15g\n", getBin(mur, i), bestmus[i]);
    }
    printf("\n\n");
    for(i = 0; i < tc.n; i++){
        printf("%.15g ", (tanh(getBin(tc,i))+1)/2);
    }
    printf("\n");
    for(i = 0; i < lN.n; i++){
        printf("%.15g ", pow(10, getBin(lN,i)));
    }
    printf("\n\n");
    for(j = 0; j < tc.n; j++){
        for(i = 0; i < lN.n; i++){
            printf("%.15g ", bestnf0s[i + lN.n*j]);
        }
        printf("\n");
    }

    return 0;
}

