#!/usr/bin/env bash

dataset=0.9
rm -rf rawSites${dataset} mitomiPSAMS mitomiE sites${dataset} alignedEM${dataset}

#extract all site data from chen dataset
#(it is normal for this to output many warnings)
mkdir -p sites${dataset}
./getSiteData.py $dataset sites${dataset} 
./mapTFgenes.py sites${dataset}

sed -ri 's/MATALPHA/MATA/' genenames
sed -ri 's/MATALPHA/MATA/' tf_gene_map
mv  sites${dataset}/MATALPHA2 sites${dataset}/MATA2

#get Cerevisiae raw sites from the above data
mkdir -p rawSites${dataset}
for i in sites${dataset}/*; do
    tf=$(basename $i)
    echo "Analyzing sites for $tf"
    ./getRawSites.py $i >rawSites${dataset}/$tf
    mkdir -p stats${dataset}/$tf
    ./regulationStats.py $i >stats${dataset}/$tf/essinfo.txt
    if [ $? -ne 0 ]; then
        rm stats${dataset}/$tf/essinfo.txt
    fi
done


#extract highest scoring mitomi EM
mkdir -p mitomiPSAMS
for tfdir in $(find mitomi -mindepth 1 -maxdepth 1 -type d); do
    tf=$(basename $tfdir | tr '[a-z]' '[A-Z]')
    if [ "$tf" = "RR" -o "$tf" = "__MACOSX" ]; then
        continue
    fi
    echo "Extracting Mitomi data for $tf"
    ./extractMitomiPSAMS.py $tfdir >mitomiPSAMS/$tf
done

#align sites to EMs
mkdir -p alignedEM${dataset}
for i in mitomiPSAMS/*; do
    tf=$(basename $i)
    if [ ! -f rawSites${dataset}/$tf ]; then
        continue
    fi
    mkdir -p stats${dataset}/$tf
    echo "Aligning $tf"
    ./alignEM.py $i rawSites${dataset}/$tf >alignedEM${dataset}/$tf
    ./makeEMFigs.py ./alignedEM${dataset}/ ./rawSites${dataset} stats${dataset}/$tf
done




