#!/usr/bin/env python2
from __future__ import with_statement
from scipy import *
from numpy import min as amin
from numpy import max as amax
import pylab
import sys, os, re, glob

class simpleLoader:
    def __init__(self, names):
        self.nucNums = -ones(256, int) 
        self.nucNums[frombuffer(names, uint8)] = arange(len(names))

    def __call__(self, seq):
        return self.nucNums[frombuffer(seq, uint8)]
seqLoad = simpleLoader('ACGT')

def getEnergies(seq, em):
    nSeq, seqLen = seq.shape
    dummyPP = arange(seqLen, dtype=int).reshape(1,seqLen).repeat(nSeq,0)
    return sum(em[dummyPP, seq], 1)

def getTheoryDistribution(em, pa=None):
    if pa == None:
        ei = mean(em, 1)
        meanE = sum(ei)
        chi = sqrt(sum(mean((em - ei[:,newaxis])**2, 1)))
        return meanE, chi
    
    pa = array(pa)/sum(pa)
    ei = sum(pa*em, 1)
    meanE = sum(ei)
    chi = sqrt(sum(sum(pa*(em - ei[:,newaxis])**2, 1)))
    return meanE, chi

def align(em, sites):
    nSeq, seqL = sites.shape
    emL, nBases = em.shape

    def sSeqs(s):
        return sites[:,max(s,0):min(s+emL,seqL)]
    
    def sEM(s):
        return em[max(-s,0):min(seqL-s,emL),:]
    
    matches = []
    for s in range(-emL, seqL+1):
        e = getEnergies(sSeqs(s), sEM(s))
        matches.append((s, mean(e)))

    m = array(matches)
    s,g = m[argmin(m[:,1]),:]
    em = sEM(s)
    meanE, chi = getTheoryDistribution(em)
    goodness = (g-meanE)/chi

    padem = r_[zeros((max(s,0),nBases)),em, zeros((max(seqL-(s+emL),0),nBases))]
    return goodness, s, padem

def getEnergy(em, site):    
    seq = seqLoad(site)
    seqlen = len(seq)
    return sum(em[arange(seqlen,dtype=int), seq])

def shiftEM(em):
    return em - mean(em,1)[:,newaxis]

def main():
    em = loadtxt(sys.argv[1])
    with open(sys.argv[2]) as f:
        sites = array([seqLoad(l.strip()) for l in f.readlines()])

    em = shiftEM(em)
    a1 = align(em, sites)
    a2 = align(em[::-1,::-1], sites)
    if a1[0] < a2[0]:
        g,shift,em = a1
    else:
        g,shift,em = a2
        shift = str(shift) + 'C'
    em = shiftEM(em)

    savetxt(sys.stdout, em)
    
if __name__ == '__main__':
    main()

