#!/usr/bin/env python2
from __future__ import with_statement
from geneParsers import loadFASTA, complement, revcomplement
import sys, os, re

def roman2int(input):
    input = input.upper()
    # map of (numeral, value, maxcount) tuples
    roman_numeral_map = (('M',  1000, 3), ('CM', 900, 1),
                        ('D',  500, 1), ('CD', 400, 1),
                        ('C',  100, 3), ('XC', 90, 1),
                        ('L',  50, 1), ('XL', 40, 1),
                        ('X',  10, 3), ('IX', 9, 1),
                        ('V',  5, 1),  ('IV', 4, 1), ('I',  1, 3))
    result, index = 0, 0
    for numeral, value, maxcount in roman_numeral_map:
        count = 0
        while input[index: index +len(numeral)] == numeral:
            count += 1 # how many of this numeral we have
            if count > maxcount:
                raise ValueError('not a valid roman numeral: %s' % input)
            result += value
            index += len(numeral)
    if index < len(input): # There are characters unaccounted for.
        raise ValueError('not a valid roman numeral: %s'%input)
    else:
        return result

def main():
    posteriorCutoff = float(sys.argv[1])
    outdir = sys.argv[2]
    distanceCutoff = 700
    
    #load genome
    genome = loadFASTA('sgd/S288C_reference_sequence_R53-1-1_20060414.fsa')

    #assume the fasta file is in order. 
    #a more serious check of this could be done.
    chromosomes = [genome[i].seq for i in range(16)]
    chrMito = genome[-1].seq
    
    #load tf site file (chen's data)
    with open('chen/all_sites_sep_cerevisiae') as f:
        lines = f.readlines()

    #load sgd feature file
    with open('sgd/SGD_features.tab.200604') as f:
        features = [l.split('\t') for l in f.readlines()]
    
    #generate mapping of genes:
    genes= {}
    for feature in features:
        (sgdid,ftype,fqual,fname,gene,alias,parent,
         sgdid2,chrm,fstart,fend,fstrand,gpos,
         cver,sver,desc) = feature
        
        if ftype == 'ORF': # and fqual == 'Verified':
            if fname in genes:
                err = "Double gene: {} :: {}".format(" ".join(feature), fname)
                raise Exception(err)
            genes[fname] = {'strand': fstrand, 
                            'start': int(fstart), 
                            'end': int(fend)}
    
    
    lastTF = 'dummy'
    nnotfound = 0
    p = 0
    while p < len(lines):
        #read info line
        pos, strand, prob, TF, neargenes, chromosomeID, abspos =lines[p].split()
        prob = float(prob)
        p += 1

        #read the seqs for other species
        seqs = {}
        while p < len(lines):
            m = re.search('(^[ACGTN]+).*\.([a-z]+)', lines[p])
            if m == None:
                break
            p += 1
            seq, species = m.groups()
            seqs[species] = seq
        
        #filter by cutoff
        if prob < posteriorCutoff:
            continue
        
        #if this is a new TF, print out the data we collected for the
        #last TF
        if lastTF != TF:
            if lastTF != 'dummy':
                with open(os.path.join(outdir, lastTF), 'wt') as f:
                    f.write("\n".join("\t".join(r) for r in regulators))
            regulators = []
            lastTF = TF
        
        #figure out location information
        if chromosomeID == 'chrMito':
            chrSeq = chrMito
        else:
            chrSeq = chromosomes[roman2int(chromosomeID[3:])-1]
        regionStart, regionEnd = [int(i) for i in abspos.split('-')]
        region = chrSeq[regionStart-1:regionEnd]
        
        siteStart, siteEnd = [int(i) for i in pos.split('-')]

        siteStart = siteStart - 1 #we want to extend by 1bp
        
        #find that sequence
        seqs['recomputed'] = region[siteStart:siteEnd]
        if strand == '-':
            seqs['recomputed'] = revcomplement(seqs['recomputed'])
        
        #revcomplement the sequences 
        #(to match chen's sequence, which is apparently rc)
        for k in seqs:
            seqs[k] = revcomplement(seqs[k])

        siteMeanPos = regionStart + (siteStart+siteEnd)/2
        
        #figure out the genes it might regulate (parse the 'neargenes' string)
        neargenes = neargenes[:-len('_S.cerevisiae')]
        seppos = neargenes.find('-Y') #there may be other dashes, 
        #                  I think only the dash followed by Y can be the split
        Lgene,Rgene = neargenes[:seppos], neargenes[seppos+1:]

        #figure out which gene it regulates, and save the information
        found = False
        if Lgene not in genes:
            print 'Warning: No such gene "' + Lgene + '"'
            LgeneData = None
        else:
            LgeneData = genes[Lgene]
            
            #COMMENTED OUT: Unecessary sanity check
            ##check that chen's region start matches the SGD region start
            ##it is often off by one for an unknown reason, but this 
            ##should have little effect
            #if LgeneData['strand'] == 'W':
            #    if regionStart-1 != LgeneData['end']:
            #        print ('Warning: L data mismatch {} -> {}: '
            #               'chen {}, sgd {} (W)'
            #              ).format(TF, Lgene, regionStart-1, LgeneData['end']) 
            #else:
            #    if regionStart-1 != LgeneData['start']:
            #        print ('Warning: L data mismatch {} -> {}: '
            #               'chen {}, sgd {} (C)'
            #              ).format(TF, Lgene, regionStart-1, LgeneData['start'])

            #filter by distance
            dist = (siteMeanPos - LgeneData['start'])
            if LgeneData['strand'] == 'C' and 0 < dist < distanceCutoff:
                regulators.append([Lgene, chromosomeID + ':' + str(siteMeanPos),
                                   str(dist), repr(seqs)])
                found = True

        if Rgene not in genes:
            print 'Warning: No such gene "' + Rgene + '"'
            RgeneData = None
        else:
            RgeneData = genes[Rgene]

            #if RgeneData['strand'] == 'W':
            #    if regionEnd+1 != RgeneData['start']:
            #        print ('Warning: R data mismatch {} -> {}: '
            #               'chen {}, sgd {} (W)'
            #              ).format(TF, Rgene, regionEnd+1, RgeneData['start']) 
            #else:
            #    if regionEnd+1 != RgeneData['end']:
            #        print ('Warning: R data mismatch {} -> {}: '
            #               'chen {}, sgd {} (C)'
            #              ).format(TF, Rgene, regionEnd+1, RgeneData['end']) 

            dist = (RgeneData['start'] - siteMeanPos)
            if RgeneData['strand'] == 'W' and 0 < dist < distanceCutoff:
                regulators.append([Rgene, chromosomeID + ':' + str(siteMeanPos), 
                                   str(dist), repr(seqs)])
                found = True
        
        #fail if the recomputed sequence does not match chen's sequence
        if seqs['recomputed'] != seqs['cerevisiae']:
            err = "Sequence Error in {}: computed: {} orig: {}".format(
                    TF, seqs['recomputed'], seqs['cerevisiae'])
            raise Exception(err)
        
        #output warnings if not found
        if not found:
            print "Warning: ",
            print "TF {} ({}) in {}-{} did not regulate any genes\n".format(
                      TF, str(prob), Lgene, Rgene),
            nnotfound += 1

    with open(os.path.join(outdir, lastTF), 'wt') as f:
            f.write("\n".join("\t".join(r) for r in regulators))

    print float(nnotfound)/len(lines)

if __name__ == '__main__':
    main()
