#!/usr/bin/env python2
import sys, os, struct
from scipy import *
import numpy as np
import pylab, matplotlib

pylab.rcParams['figure.subplot.bottom'] = 0.15
pylab.rcParams['savefig.dpi'] = 300

formatter = matplotlib.ticker.ScalarFormatter(useOffset=False)

indir = sys.argv[1]
plottingData = sys.argv[2]
outdir = sys.argv[3]

def loadGridFit(filename):
    with open(filename) as f:
        dat = f.readlines()
    fullmin = [float(x) for x in dat[0].split()]
    nonlethalmin = [float(x) for x in dat[2].split()]

    Bll = array([[float(x) for x in l.split()] for l in dat[4:24]])
    mull = array([[float(x) for x in l.split()] for l in dat[26:126]])
    
    f0 = array([float(x) for x in dat[128].split()])
    nu = array([float(x) for x in dat[129].split()])
    f0nu = array([[float(x) for x in l.split()] for l in dat[131:231]])
    return fullmin, nonlethalmin, Bll, mull, (f0,nu,f0nu)

def loadBackgroundData(filename):
    #get data used for plotting fits (histograms)
    with open(filename) as f:
        seqen = array([float(w) for w in f.readline().split()])
        prob = array([float(w) for w in f.readline().split()])
        dat = array([[float(w) for w in line.split()] for line in f.readlines()])
        xen, back = dat[:,0], dat[:,1]
        dx = xen[1] - xen[0]
        print filename, dat[1,0]- dat[0,0], dx, dat[0,0], dat[1,0]
        xbins = r_[xen - dx/2, xen[-1] + dx/2]
    return seqen, prob, dat, xen, back, dx, xbins

fullmin, nonlethalmin, Bll, mull, f0nu = loadGridFit(os.path.join(indir, 'gridfit'))
seqen, prob, dat, xen, back, dx, xbins = loadBackgroundData(plottingData)

#make 1d plot

pylab.figure(figsize=(6,3))

pylab.axes([0.17, 0.15, 0.4, 0.6])
pylab.plot(mull[:,0], mull[:,1])
pylab.xlabel(r'$\mu$')
pylab.ylabel(r'$\log \mathcal{L}$')
span = max(mull[:,1]) - min(mull[:,1])
pylab.ylim( min(mull[:,1]) - span/10.0, max(mull[:,1]) + span/10.0)
ticks, labels = pylab.xticks()

pylab.axes([0.17,0.77,0.4,0.15])
pylab.hist(seqen, bins=xbins)
pylab.xlim(mull[0,0], mull[-1,0])
pylab.xticks(ticks, ['']*len(ticks))
pylab.yticks([])
pylab.ylabel(r'$\pi(E)$')

pylab.axes([0.75,0.15,0.2,0.77])
pylab.plot(Bll[:,0], Bll[:,1])
pylab.xlabel(r'$\beta$')
pylab.ylabel(r'$\log \mathcal{L}$')
span = max(Bll[:,1]) - min(Bll[:,1])
smin, smax = min(Bll[:,1]) - span/20.0, max(Bll[:,1]) + span/20.0
pylab.vlines([1.688], smin, smax, linestyles='dashed')
pylab.ylim(smin, smax)

pylab.savefig('{}/1dfits.svg'.format(outdir))

#so is the f0-nu plot:
pylab.figure(figsize=(4.5,3))
cmax = np.max(f0nu[2].flatten())
cmin =  min(sort(f0nu[2].flatten())[-int(f0nu[2].size*0.17)], cmax-2)
pylab.pcolor(log(f0nu[1]), -log(1-f0nu[0]), f0nu[2], vmin=cmin, vmax=cmax, rasterized=True)
#pylab.plot([minmu], [minB], 'ok')
pylab.xlim(0,11)
pylab.ylim(0,10)
pylab.ylabel(r'$-\log(1-f_0)$')
pylab.xlabel(r'$\log \nu $')
c = pylab.colorbar(format=formatter)
c.set_label(r'$\log \mathcal{L}$')
c.solids.set_rasterized(True) 
c.solids.set_antialiased(False)
#ticks, labels = pylab.xticks()
pylab.ticklabel_format(useOffset=False)
pylab.subplots_adjust(right=0.8)

pylab.savefig('{}/final_f0_nu.svg'.format(outdir))

#next, find true min and plot resulting distribution

def logFitnessNonlethal(mu,B,c,en):
    return logaddexp(B*(en-mu)+log(c),0) - logaddexp(B*(en-mu),0)

def plotDists(fitness):
    pylab.figure(figsize=(3,3))
    ax = pylab.axes()

    [p.set_height(h) for p,h in zip(disthist[2][1], fithist)]

    pylab.xlabel(r'$E$')
    #pylab.ylabel('fitness')
    pylab.legend(['Data', 'Fit'])
    pylab.xlim(min(seqen), max(seqen))
    pylab.tight_layout()

def getDist(logfitness):
    fithist = exp(logfitness - max(logfitness))*back
    return fithist/(dx*sum(fithist))

pylab.figure(figsize=(6,3))
w = dx/5.0


#with open(os.path.join(indir, 'fits/fit-4d.txt')) as f:
#    fittxt = f.read()
#fit4 = [float(x) for x in fittxt.split('\n')[1].split()[:4]]
#ll4 = float(fittxt.split('\n')[1].split()[4][1:-1])

with open(os.path.join(indir, 'fits/fit-4d.binaryparam'), 'rb') as f:
    par = struct.unpack('ddddd', f.read())
    fit4 = par[:4]
    ll4 = par[4]
print fit4, ll4, logFitnessNonlethal(fit4[1], fit4[2], fit4[3], xen)
dist4 = getDist(fit4[0]*logFitnessNonlethal(fit4[1], fit4[2], fit4[3], xen))
pylab.bar(xen , dist4, w, color=(1.0,0.5,0.5))

#with open(os.path.join(indir, 'fits/fit-3d.txt')) as f:
#    fittxt = f.read()
#fit3 = [float(x) for x in fittxt.split('\n')[1].split()[:3]]
#ll3 = float(fittxt.split('\n')[1].split()[3][1:-1])
with open(os.path.join(indir, 'fits/fit-3d.binaryparam'), 'rb') as f:
    par = struct.unpack('dddd', f.read())
    fit3 = par[:3]
    ll3 = par[3]
dist3 = getDist(fit3[0]*logFitnessNonlethal(fit3[1], fit3[2], 0.99, xen))
pylab.bar(xen + w, dist3, w, color='white', hatch='//')

#with open(os.path.join(indir, 'fits/fit-exp.txt')) as f:
#    fittxt = f.read()
#fitexp = [float(x) for x in fittxt.split('\n')[1].split()[:1]]
with open(os.path.join(indir, 'fits/fit-exp.binaryparam'), 'rb') as f:
    par = struct.unpack('dd', f.read())
    fitexp = par
distexp = getDist(-fitexp[0]*xen)
pylab.bar(xen + 2*w, distexp, w, color=(0.5,0.5,1.0))

d = histogram(seqen, bins=xbins, normed=True)[0]
#pylab.bar(xen -w, d, w, color='white', yerr=sqrt(d*dx*(1-d*dx)/len(seqen)), ecolor='black')
pylab.bar(xen -w, d, w, color='white', ecolor='black')

pylab.xlabel(r'$E$')
pylab.ylabel(r'$\pi(E)$')
pylab.legend(['UFD fit', 'CFD fit', 'EXP fit', 'Data'])
pylab.xlim(min(seqen) - 2*w, max(seqen) + 3*w)
pylab.tight_layout()
pylab.savefig('{}/fit_histograms.svg'.format(outdir))

#now make plot of selection curves in log space
#pylab.figure(figsize=(4.2,3))
pylab.figure(figsize=(4.2,3))
pylab.gca().locator_params(tight=True, nbins=6)
boundhist = histogram(seqen, bins=xbins)[0]
boundhist = boundhist/(sum(boundhist)*dx)
#print sum(back)*dx
lpi_pi0 = log(boundhist/back)
boundhisterr = sqrt(boundhist*dx*(1-boundhist*dx)/len(seqen))
lpi_pi0_err = boundhisterr/(boundhist*dx)
dataline = pylab.errorbar(xen, lpi_pi0, fmt='.', color='black', yerr=lpi_pi0_err, ecolor='black')[0]
#pylab.scatter(xen, lpi_pi0, c='black')
finex = linspace(min(seqen)-3,max(seqen)+1,1000)

nulogf = fit4[0]*logFitnessNonlethal(fit4[1],fit4[2],fit4[3],xen)
logZ = max(nulogf) + log(sum(back*exp(nulogf - max(nulogf)))) + log(dx)
UFDline = pylab.plot(finex, fit4[0]*logFitnessNonlethal(fit4[1],fit4[2],fit4[3],finex) - logZ, 'r-')[0]

nulogf = fit3[0]*logFitnessNonlethal(fit3[1],fit3[2],0.99,xen)
logZ = max(nulogf) + log(sum(back*exp(nulogf - max(nulogf)))) + log(dx)
CFDline = pylab.plot(finex, fit3[0]*logFitnessNonlethal(fit3[1],fit3[2],0.99,finex) - logZ, 'k--')[0]

nulogf = -xen*fitexp[0]
logZ = max(nulogf) + log(sum(back*exp(nulogf - max(nulogf)))) + log(dx)
expline = pylab.plot(finex, -finex*fitexp[0] - logZ, 'g--')[0]

#offsets = fit4[0]*logFitnessNonlethal(fit4[1],fit4[2],fit4[3],xen) - lpi_pi0
#offsets = offsets[~isnan(offsets) & ~isinf(offsets)]
#shift = mean(offsets)
#pylab.plot(finex, fit4[0]*logFitnessNonlethal(fit4[1],fit4[2],fit4[3],finex) - shift, 'r-')

#offsets = fit3[0]*logFitnessNonlethal(fit3[1],fit3[2], 0.9,xen) - lpi_pi0
#offsets = offsets[~isnan(offsets) & ~isinf(offsets)]
#shift = mean(offsets)
#pylab.plot(finex, fit3[0]*logFitnessNonlethal(fit3[1],fit3[2],0.9,finex) - shift, 'k--')

#offsets = -xen*fitexp[0] - lpi_pi0
#offsets = offsets[~isnan(offsets) & ~isinf(offsets)]
#shift = mean(offsets)
#pylab.plot(finex, -finex*fitexp[0] - shift, 'g--')

pylab.ylabel(r'$\log \frac{\pi(E)}{\pi_0(E)}$')
pylab.xlabel(r'$E$')
pylab.xlim(min(seqen)*1.1, max(seqen)*0.9)
pylab.ylim(min(lpi_pi0[~isinf(lpi_pi0) & ~isnan(lpi_pi0)])*0.8, max(lpi_pi0)*1.5)

pylab.legend([UFDline, CFDline, expline, dataline], ['UFD fit', 'CFD fit', 'EXP fit', 'Data'], prop={'size': 10})
pylab.savefig('{}/selection.svg'.format(outdir))
    
#pylab.show()

