#!/usr/bin/env python2
from scipy import *
import numpy as np
import sys, os
import subprocess
from getDist import getEDist, resampleDist, getPaDpa
from load import loadSites, getEnergies, getFreqs, showFreqs, showEnergies
from scipy.stats.distributions import binom, norm
import pylab, matplotlib
import matplotlib as mpl
from matplotlib.image import BboxImage
from matplotlib.transforms import Bbox, TransformedBbox, \
       blended_transform_factory, Affine2D, ScaledTranslation
from mpl_toolkits.axes_grid.inset_locator import inset_axes

mpl.rc('text', usetex=False)
mpl.rc('font', **{'family':'sans-serif', 'sans-serif': 'Arial', 'serif': 'Times New Roman'})



def nrm(x,dx):
    return x/(sum(x)*dx)

fig = pylab.figure(figsize=(10,6))
fig.subplots_adjust(left=0.1, right=0.95, bottom=0.1, top=0.95, hspace=0.4, wspace=0.25)
pylab.title('REB1')

#first, plot logos:
sites, info = loadSites('../rawData/rawSites0.9/REB1', names="ACGT")
em = loadtxt('../rawData/alignedEM0.9/REB1')
emfreq = exp(-1.688*em)
emfreq = emfreq/sum(emfreq,1)[:,newaxis]

pylab.subplot(6,2,2)
pylab.text(-0.15,0.95,'B', fontsize=16, transform=pylab.gca().transAxes, family='serif')
pylab.title('Energy Matrix', fontsize=10)
showEnergies(-em, fig=fig)
pylab.xticks([])
pylab.yticks([-2,2])
pylab.ylabel('')
pylab.subplot(6,2,4)
pylab.title('EM logo', fontsize=10)
showFreqs(emfreq, "ACGT", em.shape[0], fig)
pylab.xticks([])
pylab.yticks([0,1,2])
pylab.ylabel('')
pylab.subplot(6,2,6)
pylab.title('Sequence logo', fontsize=10)
showFreqs(getFreqs(sites, 4), fig=fig)
pylab.xticks([])
pylab.yticks([0,1,2])
pylab.ylabel('')

#next, plot nice fit:
def logFitnessNonlethal(mu,B,c,en):
    return logaddexp(B*(en-mu)+log(c),0) - logaddexp(B*(en-mu),0)
def getDist(logfitness):
    fithist = exp(logfitness - max(logfitness))*back
    return fithist/(dx*sum(fithist))

with open('../processedData/chenDataUnnormedPlotting0.9/REB1') as f:
    seqen = array([float(w) for w in f.readline().split()])
    prob = array([float(w) for w in f.readline().split()])
    dat = array([[float(w) for w in line.split()] for line in f.readlines()])
    xen, back = dat[:,0], dat[:,1]
    dx = xen[1] - xen[0]
    xbins = r_[xen - dx/2, xen[-1] + dx/2]

pylab.subplot(223)
pylab.text(-0.2,0.95,'C', fontsize=16, transform=pylab.gca().transAxes, family='serif')
#pylab.title('Bound site energy distribution')
w = dx/5.0

outdir = 'data/REB1/fits'
with open(outdir + '/fit-4d.txt') as f:
    fittxt = f.read()
fit4 = [float(x) for x in fittxt.split('\n')[1].split()[:4]]
ll4 = float(fittxt.split('\n')[1].split()[4][1:-1])
dist4 = getDist(fit4[0]*logFitnessNonlethal(fit4[1], fit4[2], fit4[3], xen))
pylab.bar(xen , dist4, w, color=(1.0,0.5,0.5))

with open(outdir + '/fit-3d.txt') as f:
    fittxt = f.read()
fit3 = [float(x) for x in fittxt.split('\n')[1].split()[:3]]
ll3 = float(fittxt.split('\n')[1].split()[3][1:-1])
dist3 = getDist(fit3[0]*logFitnessNonlethal(fit3[1], fit3[2], 0.99, xen))
pylab.bar(xen + w, dist3, w, color='white', hatch='//')

with open(outdir + '/fit-exp.txt') as f:
    fittxt = f.read()
fitexp = [float(x) for x in fittxt.split('\n')[1].split()[:1]]
distexp = getDist(-fitexp[0]*xen)
pylab.bar(xen + 2*w, distexp, w, color=(0.5,0.5,1.0))

d = histogram(seqen, bins=xbins, normed=True)[0]
#pylab.bar(xen -w, d, w, color='white', yerr=sqrt(d*dx*(1-d*dx)/len(seqen)), ecolor='black')
pylab.bar(xen -w, d, w, color='white', ecolor='black')

pylab.xlabel(r'$E$')
pylab.ylabel(r'$\pi(E)$')
pylab.legend(['Fermi-Dirac Fit', 'Nonlethal Fit', 'Exponential Fit', 'Data'], prop={'size': 10})
pylab.xlim(min(seqen) - 2*w, max(seqen) + 3*w)

#plot neutral distributions
cdict = {'red':   ((0.0,0.0,0.0),
                   (0.5,0.0,0.0),
                   (1.0,1.0,1.0)), 
         'green': ((0.0,1.0,1.0),
                   (0.5,0.0,0.0),
                   (1.0,0.0,0.0)), 
         'blue':  ((0.0,0.0,0.0),
                   (0.5,1.0,1.0),
                   (1.0,0.0,0.0))}
rbcmap = matplotlib.colors.LinearSegmentedColormap('redblue',cdict,256)
ax = pylab.subplot(221)
pylab.text(-0.2,0.95,'A', fontsize=16, transform=pylab.gca().transAxes, family='serif')
dat64 = loadtxt('../processedData/dists64_0.9/REB1')
x64 = dat64[:,0]
dx64 = dat64[1,0] - dat64[0,0]
xbins64 = r_[x64[0]-dx64/2, x64+dx64/2]
n,i = nrm(dat64[:,1],dx64), nrm(dat64[:,2],dx64)#neutral, intergenic

h = histogram(seqen, bins=xbins64, normed=True)[0]# boundhist

pylab.semilogy(x64, n, '-k')
pylab.semilogy(x64, i, '--k')
trans = blended_transform_factory(ax.transData, ax.transAxes)

pctdev = (i-n)/n
vmax = 50
vmin = -vmax
pctdev = pctdev + 0.5
colors = [rbcmap(v) if not isnan(v) else (1,1,1,1) for v in pctdev]
t = Affine2D([[1,0,0],[0,0.05,0], [0,0,0]]) + trans
pylab.bar(x64-dx64/2, ones(len(x64)), width=dx64+0.02, transform=t, color=rbcmap(pctdev), bottom=0, ec='none', lw=0, aa=False)

t = Affine2D([[1,0,0],[0,0.2/np.max(h),0.05], [0,0,1]]) + trans
pylab.bar(x64-dx64/2, h, width=dx64, transform=t, color='w', bottom=0)

pylab.xlim(xbins64[0],xbins64[-1])
pylab.ylim(0,0.2)
pylab.ylim(1e-6,0.2)
pylab.ylabel('Probability')
pylab.xlabel('$E$')
#pylab.title('Neutral and Intergenic Distributions')

sm = matplotlib.cm.ScalarMappable(cmap=rbcmap, norm=matplotlib.pyplot.normalize(vmin=vmin, vmax=vmax))
sm._A = []
cax = pylab.axes((0.2, 0.66, 0.10, 0.02))
cax.set_xticks([vmin, 0, vmax])
cax.set_xticks([])
cax.set_yticks([])
pylab.colorbar(sm, cax=cax, orientation='horizontal', ticks=[vmin, 0, vmax], format='%.3g')



#plot 'selection' curve
pylab.subplot(224)
pylab.text(-0.15,0.95,'D', fontsize=16, transform=pylab.gca().transAxes, family='serif')

boundhist = histogram(seqen, bins=xbins)[0]
boundhist = boundhist/(sum(boundhist)*dx)
lpi_pi0 = log(boundhist/back)
boundhisterr = sqrt(boundhist*dx*(1-boundhist*dx)/len(seqen))
lpi_pi0_err = boundhisterr/(boundhist*dx)
dataline = pylab.errorbar(xen, lpi_pi0, fmt='.', color='black', yerr=lpi_pi0_err, ecolor='black')[0]
finex = linspace(min(seqen)-1,max(seqen)+1,1000)

with open('../processedData/chenDataUnnormedPrecise0.9/REB1') as f:
    seqen2 = array([float(w) for w in f.readline().split()])
    prob2 = array([float(w) for w in f.readline().split()])
    dat2 = array([[float(w) for w in line.split()] for line in f.readlines()])
    xen2, back2 = dat2[:,0], dat2[:,1]
    dx2 = xen[1] - xen2[0]
    xbins2 = r_[xen2 - dx2/2, xen2[-1] + dx2/2]

Z = sum(back2*exp(fit4[0]*logFitnessNonlethal(fit4[1],fit4[2],fit4[3],xen2)))*dx2
UFDline = pylab.plot(finex, fit4[0]*logFitnessNonlethal(fit4[1],fit4[2],fit4[3],finex) - log(Z), 'r-')[0]

Z = sum(back2*exp(fit3[0]*logFitnessNonlethal(fit3[1],fit3[2],0.99,xen2)))*dx2
CFDline = pylab.plot(finex, fit3[0]*logFitnessNonlethal(fit3[1],fit3[2],0.99,finex) - log(Z), 'k--')[0]

Z = sum(back2*exp(-xen2*fitexp[0]))*dx2
expline = pylab.plot(finex, -finex*fitexp[0] - log(Z), 'g--')[0]

pylab.ylabel(r'$\log \frac{\pi(E)}{\pi_0(E)}$')
pylab.xlabel(r'$E$')
pylab.xlim(min(seqen)*1.05, max(seqen)*0.9)
pylab.ylim(min(lpi_pi0[~isinf(lpi_pi0) & ~isnan(lpi_pi0)])*0-2, max(lpi_pi0)*1.5)

pylab.legend([UFDline, CFDline, expline, dataline], ['UFD fit', 'CFD fit', 'EXP fit', 'Data'], prop={'size': 10})

pylab.show()


