#!/usr/bin/env python2
import glob, sys, os, shutil, re
from scipy import *
import load
import pylab
import latex2svg

datadir = sys.argv[1]

gridData = False

def AIC(k,n,lL):
    return 2*(k - lL) + float(2*k*(k+1))/(n-k-1)

def writelatex(tex):
    tex = '${}$'.format(tex)
    fn = latex2svg.svgfilename(latex2svg.latexhash(tex))
    if not os.path.exists('lateximg'):
        os.mkdir('lateximg')
    fp = os.path.join('lateximg', fn)
    if not os.path.exists(fp):
        svg = latex2svg.makeLatex(tex)
        with open(fp, 'wt') as f:
            f.write(svg)
    else:
        with open(fp, 'rt') as f:
            svg = f.read()
    return svg.strip()

paramNames = [writelatex(x) for x in [r'\nu', r'\mu', r'\beta', 'f_0']]
nutxt, mutxt, betatxt, f0txt = paramNames
gammatxt = writelatex(r'\gamma')
alphatxt = writelatex(r'\alpha')
lltxt = writelatex(r'\log \mathcal{L}')

tfdirs = [os.path.join(datadir, tf) for tf in os.walk(datadir).next()[1]]

#sort them by nUniqueSeq
nSeqs = {}
for tfdir in tfdirs:
    tf = os.path.basename(tfdir)
    with open(os.path.join('../rawData/stats0.9/', tf, 'stats.txt')) as f:
        lines = f.readlines()
        nSeqs[tfdir] = (int(lines[0].split()[1]), int(lines[1].split()[1]))
#print >>sys.stderr, tfdirs, [nSeqs[dr] for dr in tfdirs]
tfdirs.sort(key=lambda x: nSeqs[x], reverse=True)
def cc(a,b):
    c = cmp(nSeqs[a][1], nSeqs[b][1])
    if c != 0:
        return c
    return cmp(nSeqs[a][0], nSeqs[b][0])
tfdirs.sort(cmp=cc, reverse=True)
tfdirs = tfdirs[:12]
#print >>sys.stderr, tfdirs, [nSeqs[dr] for dr in tfdirs]

def div(*e, **ks):
    return "<div {}>{}</div>".format(ks.get('attr', ""), "".join(map(str, e)))
def td(*e, **ks):
    return "<td {}>{}</td>".format(ks.get('attr', ""), "".join(map(str, e)))
def th(*e):
    return "<th>{}</th>".format("".join(map(str, e)))
def tr(*e):
    return "<tr>{}</tr>".format("".join(map(str, e)))
def table(*e, **ks):
    return "<table {}>{}</table>".format(ks.get('attr', ""), "".join(map(str, e)))

dat = []
def printRow(tfdir):
    tf = os.path.basename(tfdir)
    #print >>sys.stderr, tf

    with open(os.path.join('../rawData/stats0.9/', tf, 'stats.txt')) as f:
        nSeq, nUnique, emEntr, siteEntropy, kldiff = [l.split()[1] for l in f.readlines()]
        nSeq = int(nSeq)

    try:
        with open(os.path.join('../rawData/stats0.9/', tf, 'essinfo.txt')) as f:
            txt = f.readlines()
            isEssential = txt[0].split()[2] == 'essential'
            nRegEssential = txt[1].split()[0]
            nRegNonessential = txt[2].split()[0]
            nRegUnknown = txt[3].split()[0]
        esstxt = "Genes Regulated:<BR>{} essential<BR>{} nonessential<BR>{} unknown".format(
                  nRegEssential, nRegNonessential, nRegUnknown)
    except:
        esstxt = ""
    tftxt = tf + "<BR>({}&nbsp;TF)".format('essential' if isEssential else 'nonessential') + \
                 "<BR><BR>Total&nbsp;sites:&nbsp;{}<BR>Unique&nbsp;sites:&nbsp;{}\n\n".format(nSeq, nUnique)

    with open(os.path.join(datadir, tf, 'gridfit')) as f:
        txt = f.readlines()
        fit = [float(x) for x in txt[0].split()]
        ll4 = fit[-1] 
        fit4 = fit[:4]
        gamma4 = fit4[0]*(1-fit4[3])
        aic4t = "{:.7g}".format(AIC(4, nSeq, ll4))

        fit4t = ["{:.3g}".format(x) for x in fit4]
        gamma4t = "{:.3g}".format(gamma4)
        ll4t = "{:.7g}".format(ll4)

        fit = [float(x) for x in txt[2].split()]
        ll3 = fit[-1] 
        fit3 = fit[:3]
        gamma3 = fit3[0]*(1-0.99)
        aic3t = "{:.7g}".format(AIC(3, nSeq, ll3))

        fit3t = ["{:.3g}".format(x) for x in fit3]
        gamma3t = "{:.3g}".format(gamma3)
        ll3t = "{:.7g}".format(ll3)

    with open(os.path.join(datadir, tf, 'fits/fit-exp.txt')) as f:
        txt = f.readlines()
        fit = [float(x) for x in txt[1].replace('(', '').replace(')', '').split()]
        llexp = -fit[-1]
        fitexp = -fit[0]
        derivsexp = eval(txt[4])
        aicexpt = "{:.7g}".format(AIC(1, nSeq, llexp))

        fitexpt = "{:.3g}".format(fitexp)
        llexpt = "{:.7g}".format(llexp)

        fit3txt = "\n".join("{}: {}".format(a,b) for a,b in zip(paramNames[:3], fit3)) + '\n\n{}: {}'.format(gammatxt, gamma3) + '\n\n{}: -{}'.format(lltxt, ll3)

    paramtable = table(tr(th(""), th("UFD"), th("CFD"), th("EXP")),
                       tr(td(mutxt),    td(fit4t[1]), td(fit3t[1]), td("")),
                       tr(td(betatxt),  td(fit4t[2]), td(fit3t[2]), td("")),
                       tr(td(nutxt),    td(fit4t[0]), td(fit3t[0]), td("")),
                       tr(td(f0txt),    td(fit4t[3]), td("(0.99)"),  td("")),
                       tr(td(gammatxt), td(gamma4t),  td(gamma3t),  td("")),
                       tr(td(alphatxt), td(""),       td(""),       td(fitexpt)), 
                       tr(td(lltxt),    td(ll4t),     td(ll3t),     td(llexpt)), 
                       tr(td("AIC"),    td(aic4t),    td(aic3t),    td(aicexpt)), 
                       attr='class="params"')

    return tr(td(tftxt), 
              td('<img height="250px" src="{}/ivsn.svg">'.format(tf)),
              td('<img height="250px" src="{}/Logos.svg">'.format(tf)), 
              td(paramtable), 
              td('<img height="250px" src="{}/fit_histograms.svg">'.format(tf)), 
              td('<img height="250px" src="{}/selection.svg">'.format(tf)), 
              td('<img height="250px" src="{}/1dfits.svg">'.format(tf)), 
              td('<img height="250px" src="{}/final_f0_nu.svg">'.format(tf))
            )

print """<!DOCTYPE html>
<html>
<head>
<style>
img{
}
td{
    border-width: 1px;
    border-style: solid;
}
table {
    border:none;
    border-collapse: collapse;
}

table td{
    border-left: none;
    border-right: none;
}

table.params{
    pargin: 10px;
}

table.params td{
    border: none;
    border-right: solid;
    border-width: 1px;
    padding: 1px 5px 1px 20px;
    text-align: right;
}

table.params th{
    border: none;
    border-right: solid;
    border-bottom: solid;
    border-width: 1px;
    text-align: right;
    padding: 1px 5px 1px 20px;
}


</style>
</head>
<body>
"""

header =  tr(th('(A) Dataset'), 
             th('(B) Raw Energies'),
             th('(C) Logos'), 
             th('(D) Fit Parameters'), 
             th('(E) Fit Distributions'), 
             th('(F) Log Fitness'), 
             th('(G) {} and {} Likelihood Functions').format(mutxt, betatxt), 
             th('(H) {}-{} Likelihood Function'.format(nutxt, f0txt))
            )

print table(*([header] + [printRow(fn) for fn in tfdirs]))

print """
</body>
</html>
"""
